/*
 * Copyright 2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Id: CreateComponentCommand.java 420313 2006-07-09 16:42:46Z gvanmatre $
 */
package org.apache.shale.clay.component.chain;

import java.util.Map;

import javax.faces.component.UIComponent;
import javax.faces.context.FacesContext;
import javax.faces.el.ValueBinding;

import org.apache.commons.chain.Context;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shale.clay.component.Clay;
import org.apache.shale.clay.config.Globals;
import org.apache.shale.clay.config.beans.AttributeBean;
import org.apache.shale.clay.config.beans.Attributes;
import org.apache.shale.clay.config.beans.ComponentBean;

/**
 * <p>
 * This <code>Command</code> is used to build <strong>parent</strong> and
 * <strong>child</strong> <code>UIComponent</code>'s from the
 * <code>displayElement</code> in the {@link org.apache.shale.clay.component.chain.ClayContext}.
 * </p>
 */
public class CreateComponentCommand extends AbstractCommand {

    /**
     * <p>
     * Common Logger utility.
     * </p>
     */
    private static Log log;
    static {
        log = LogFactory.getLog(CreateComponentCommand.class);
    }

    /**
     * <p>Tests to see if myfaces runtime is installed by trying to load a known class.</p>
     *
     * @return <code>true</code> if myfaces runtime is loaded
     */
    private boolean isMyfacesRuntimeInstalled() {
        try {
            Class.forName("org.apache.myfaces.application.jsp.JspViewHandlerImpl");
            return true;
        } catch (ClassNotFoundException e) {
            return false;
        }
    }


    /**
     * <p>Returns a unique component id for the request/response lifecycle.</p>
     *
     * @param facesContext faces context
     * @return unique component id
     */
    private String createUniqueId(FacesContext facesContext) {
        String id = null;

        if (isMyfacesRuntimeInstalled()) {
            Map requestMap = facesContext.getExternalContext().getRequestMap();
            SequenceGenerator generator = (SequenceGenerator) requestMap.get(Globals.CLAY_SEQUENCE_GENERATOR);
            if (generator == null) {

                generator = new SequenceGenerator();
                requestMap.put(Globals.CLAY_SEQUENCE_GENERATOR, generator);
            }
            id = generator.createUniqueId();
        } else {
            id = facesContext.getViewRoot().createUniqueId();
        }

        return id;
    }


    /**
     * <p>
     * Creates a new faces component from the component metadata.
     * </p>
     * @param context common chains
     * @return <code>true</code> if the chain is complete
     * @exception Exception propagated up to the top of the chain
     */
    public boolean execute(Context context) throws Exception {

        boolean isFinal = true;

        ClayContext clayContext = (ClayContext) context;
        if (clayContext == null) {
            throw new NullPointerException(getMessages()
                    .getMessage("clay.null.clayContext"));
        }

        ComponentBean displayElement = clayContext.getDisplayElement();
        if (displayElement == null) {
            throw new NullPointerException(getMessages()
                    .getMessage("clay.null.componentBean"));
        }

        UIComponent parent = clayContext.getParent();
        if (parent == null) {
            throw new NullPointerException(getMessages()
                    .getMessage("clay.null.parentBean"));
        }

        FacesContext facesContext = clayContext.getFacesContext();
        if (facesContext == null) {
            throw new NullPointerException(getMessages()
                    .getMessage("clay.null.facesContext"));
        }

        // create a new scoped symbol table
        Map symbolTable = new Attributes();
        // inherit the parents symbols
        symbolTable.putAll(clayContext.getSymbols());
        // override config (XML, HTML) symbols
        symbolTable.putAll(displayElement.getSymbols());
        // push to context
        clayContext.setSymbols(symbolTable);

        // evaluate nested symbols; symbols having symbols as values
        realizeSymbols(clayContext);


        String id = displayElement.getId();
        if (id == null) {
            id = createUniqueId(facesContext);
        } else {
            id = replaceMnemonic(clayContext, id);
        }

        //Check to see if the replacement failed.  This can happen if the
        //symbol is missing.  The id will still containing the symbol delimiter
        //character.  The other scenario is if the value of the symbol is null.

        if (id == null || id.indexOf('@') > -1) {

           if (id == null) {
              id = displayElement.getId();
           }

           throw new RuntimeException(getMessages().getMessage("create.component.invalid.id",
                   new Object[] {id, symbolTable}));
        }

        UIComponent child = null;
        if (displayElement.getFacetName() != null) {
            child = parent.getFacet(displayElement.getFacetName());
        } else {
            child = parent.findComponent(id);
        }

        if (child == null) {
            try {
                AttributeBean attr = displayElement.getAttribute("binding");
                if (attr != null
                    && attr.getValue() != null) {

                   clayContext.setAttribute(attr);
                   String expr = replaceMnemonic(clayContext);
                   ValueBinding vb = facesContext.getApplication().createValueBinding(expr);
                   child = facesContext.getApplication().createComponent(vb, facesContext,
                                                                    displayElement.getComponentType());
                   child.setValueBinding("binding", vb);

                } else {

                   child = facesContext.getApplication().createComponent(
                        displayElement.getComponentType());
                }

            } catch (Exception e) {
                log.error(getMessages().getMessage("create.component.error",
                        new Object[] { displayElement }), e);
                        throw e;
            }

            child.setId(id);
            if (displayElement.getFacetName() != null) {
                parent.getFacets().put(replaceMnemonic(clayContext, displayElement.getFacetName()), child);

                if (log.isDebugEnabled()) {
                    log.debug(getMessages().getMessage("create.facet.component",
                        new Object[] { id, displayElement.getJsfid()}));
                }

            } else {
                parent.getChildren().add(clayContext.getChildIndex(), child);

                if (log.isDebugEnabled()) {
                    log.debug(getMessages().getMessage("create.component",
                        new Object[] { id, displayElement.getJsfid(), new Integer(clayContext.getChildIndex()) }));
                }

            }

            if (child instanceof Clay) {
                //save the display element used to create the component for exception
                //reporting when the jsfid is null.  This should only happen when using
                //symbol replacement of a nested clay component with HTML views
                child.getAttributes().put(Globals.CLAY_RESERVED_ATTRIBUTE, displayElement);
            }

            // continue with the addComponent chain
            isFinal = false;
        } else {
            if (log.isDebugEnabled()) {
                log.debug(getMessages().getMessage("create.component.exists",
                    new Object[] { id, displayElement.getJsfid(), new Integer(clayContext.getChildIndex()) }));
            }
        }

        // if target is a Clay component it might contain symbols
        if (child instanceof Clay) {
            // override symbols from nested clay component
            symbolTable.putAll(((Clay) child).getSymbols());
            // capture current symbols for the root of the nested component
            ((Clay) child).getSymbols().putAll(symbolTable);

            // push to context
            clayContext.setSymbols(symbolTable);

            // evaluate nested symbols
            realizeSymbols(clayContext);
        }


        // reassign the child to the converter for the
        // AssignPropertiesCommand
        clayContext.setChild(child);

        return isFinal;
    }


}
