/*
 * Copyright 2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Id: ClayTemplateParser.java 425271 2006-07-25 04:34:13Z gvanmatre $
 */
package org.apache.shale.clay.config;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Iterator;
import java.util.List;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shale.clay.config.beans.ComponentBean;
import org.apache.shale.clay.config.beans.ComponentConfigBean;
import org.apache.shale.clay.config.beans.ConfigBean;
import org.apache.shale.clay.config.beans.ElementBean;
import org.apache.shale.clay.config.beans.TemplateConfigBean;
import org.apache.shale.clay.parser.Node;
import org.apache.shale.clay.parser.Parser;
import org.apache.shale.clay.parser.builder.Builder;
import org.apache.shale.util.Messages;
import org.xml.sax.SAXException;

/**
 * <p>
 * This class is responsible for loading an HTML template into a graph of
 * {@link ComponentBean}'s that represents a JSF component tree. It is used by
 * {@link org.apache.shale.clay.config.beans.TemplateConfigBean}, a subclass of
 * {@link org.apache.shale.clay.config.beans.ComponentConfigBean}.
 * </p>
 */
public class ClayTemplateParser implements ClayConfigParser {

    /**
     * <p>
     * Commons logging utility object static instance.
     * </p>
     */
    private static Log log;
    static {
        log = LogFactory
                .getLog(org.apache.shale.clay.config.ClayXmlParser.class);
    }

    /**
     * <p>
     * Message resources for this class.
     * </p>
     */
    private static Messages messages = new Messages(
            "org.apache.shale.clay.Bundle", ClayConfigureListener.class
                    .getClassLoader());

    /**
     * <p>
     * Object pool for HTML template configuration files.
     * </p>
     */
    private ConfigBean config = null;

    /**
     * <p>
     * Sets an object pool for HTML template configuration files.
     * </p>
     *
     * @param config
     *            handler
     */
    public void setConfig(ConfigBean config) {
        this.config = config;
    }

    /**
     * <p>
     * Returns an object pool for HTML template configuration files.
     * </p>
     *
     * @return config handler
     */
    public ConfigBean getConfig() {
        return config;
    }

    /**
     * <p>
     * Loads the <code>templateURL</code> identified by the
     * <code>templateName</code> into a graph of {@link ComponentBean}'s.
     * </p>
     *
     * @param templateURL
     *            template file
     * @param templateName
     *            jsfid
     * @exception SAXException
     *                XML parse error
     * @exception IOException
     *                XML parse error
     */
    public void loadConfigFile(URL templateURL, String templateName)
            throws IOException, SAXException {

        ((ComponentConfigBean) config).addChild(generateElement(templateURL,
                templateName));
    }

    /**
     * <p>
     * Loads the template file and parses it into a composition of metadata
     * that's used by the {@link org.apache.shale.clay.component.Clay}
     * component. This metadata is used to construct a JSF subtree within target
     * view.
     * </p>
     *
     * @param templateURL
     *            template file
     * @param templateName
     *            jsfid
     * @return config bean graph holding the template file
     * @exception IOException
     *                loading template file
     */
    protected ComponentBean generateElement(URL templateURL, String templateName)
            throws IOException {

        if (log.isInfoEnabled()) {
            log.info(messages.getMessage("loading.template",
                    new Object[] { templateName }));
        }

        ComponentBean root = new ComponentBean();
        root.setJsfid(templateName);
        root.setComponentType("javax.faces.NamingContainer");

        // generate the document

        InputStream inputStream = null;
        StringBuffer buffer = null;

        try {
            inputStream = templateURL.openStream();
            buffer = loadTemplate(inputStream, templateName);
        } finally {
            if (inputStream != null) {
                inputStream.close();
            }
        }

        List roots = new Parser().parse(buffer);
        Iterator ri = roots.iterator();
        while (ri.hasNext()) {
            Node node = (Node) ri.next();
            Builder renderer = node.getBuilder();
            ElementBean child = renderer.createElement(node);

            root.addChild(child);
            if (renderer.isChildrenAllowed()) {
                renderer.encode(node, child, child);
            } else {
                renderer.encode(node, child, root);
            }
        }

        roots.clear();
        roots = null;
        buffer.setLength(0);
        buffer = null;
        ri = null;

        // verify there is not a duplicate component id within a naming
        // container.
        config.checkTree(root);

        // compress the tree merging adjacent verbatim nodes
        if (config instanceof TemplateConfigBean) {
            ((TemplateConfigBean) config).optimizeTree(root);
        }

        return root;
    }

    /**
     * <p>
     * Loads the template into a <code>StringBuffer</code> using the
     * <code>inputStream</code> and <code>templateName</code> parameters.
     * </p>
     *
     * @param inputStream
     *            template file
     * @param templateName
     *            jsfid
     * @return loaded template file
     * @exception IOException
     *                loading template file
     */
    protected StringBuffer loadTemplate(InputStream inputStream,
            String templateName) throws IOException {

        StringBuffer buff = new StringBuffer();

        try {
            int c = 0;
            done: while (true) {
                c = inputStream.read();
                if (c > -1) {
                    buff.append((char) c);
                } else {
                    break done;
                }

            }
        } catch (IOException e) {
            log.error(messages.getMessage("loading.template.exception",
                    new Object[] { templateName }), e);
            throw e;
        }

        return buff;

    }

}
