/*
 * Copyright 2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Id: ConfigDefinitionsWatchdogFilter.java 420313 2006-07-09 16:42:46Z gvanmatre $
 */
package org.apache.shale.clay.config.beans;

import org.apache.commons.chain.Context;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.shale.application.AbstractRegExpFilter;
import org.apache.shale.faces.ShaleWebContext;

/**
 * <p>This is the timing mechanism for looking for modified
 * Clay templates and configuration files.  This is a preprocess
 * filter chains command that should only be used in the
 * development environment.</p>
 */
public class ConfigDefinitionsWatchdogFilter extends AbstractRegExpFilter {

    /**
     * <p>Log instance for this class.</p>
     */
    private static Log log = LogFactory.getLog(ConfigDefinitionsWatchdogFilter.class);


    /**
     * <p>Return the servlet path (if any) concatenated with the path info
     * (if any) for this request.</p>
     *
     * @param context <code>Context</code> for the current request
     * @return servletPath and pathInfo
     */
    protected String value(ShaleWebContext context) {

        String servletPath = context.getRequest().getServletPath();
        if (servletPath == null) {
            servletPath = "";
        }
        String pathInfo = context.getRequest().getPathInfo();
        if (pathInfo == null) {
            pathInfo = "";
        }
        return servletPath + pathInfo;

    }


    /**
     * <p>Perform the matching algorithm against the value
     * returned by the <code>value()</code> method.  If the
     * value <code>matches()</code> the <code>includes</code>
     * pattern list, the {@link org.apache.shale.clay.component.Clay}
     * configuration files are checked for changes and reloaded.</p>
     *
     * @param context <code>ShaleWebContext</code> for this request
     * @return <code>true</code> if the chain is done
     * @exception Exception thrown back to the calling command
     */
    public boolean execute(Context context) throws Exception {

        // Acquire the value to be tested
        ShaleWebContext webContext = (ShaleWebContext) context;
        String value = value(webContext);
        if (log.isDebugEnabled()) {
            log.debug("execute(" + value + ")");
        }

        // Check for a match on the included list
        if (value != null && matches(value, getIncludesPatterns(), true)) {
            if (log.isTraceEnabled()) {
                log.trace("  accept(include)");
            }
            accept(webContext);
        }

        return false;

    }


    /**
     * <p>Trigger reloading of the {@link org.apache.shale.clay.component.Clay}'s
     * xml configuration files if the
     * <code>org.apache.shale.clay.AUTO_RELOAD_CONFIG_FILES</code> init
     * parameter is set to <code>true</code> in the web.xml.  The HTML templates
     * are re-cashed on-demand due to their atomicity.  The XML configuration
     * files are shared by all {@link org.apache.shale.clay.component.Clay}
     * view composition mechanisms so all files must be reloaded if a change
     * is made.</p>
     *
     * @param context <code>Context</code> for the current request
     * @exception Exception thrown back to the caller
     */
    protected void accept(ShaleWebContext context) throws Exception {

        ConfigBeanFactory.refresh();
    }


}
