/*
 * Copyright 2005 The Apache Software Foundation.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/*
 * $Id: DirectiveBuilderRule.java 420313 2006-07-09 16:42:46Z gvanmatre $
 */
package org.apache.shale.clay.parser.builder.chain;

import org.apache.commons.chain.Command;
import org.apache.commons.chain.Context;
import org.apache.shale.clay.parser.Node;
import org.apache.shale.clay.parser.builder.Builder;
import org.apache.shale.clay.parser.builder.IgnoreBuilder;
import org.apache.shale.clay.parser.builder.VoidBuilder;

/**
 * <p>This builder will be ordered first in the rule registration list.
 * If the node has a <code>jsfid</code> attribute, and its value is
 * "ignore", or "void" then special assumptions will be made.</p>
 */
public class DirectiveBuilderRule implements Command {

    /**
     * <p>The builders that will assined to a "ignore" or "void"
     * {@link Builder}.
     * <p>
     */
    private static final Builder[] BUILDERS = {new IgnoreBuilder(), new VoidBuilder()};

    /**
     * <p>If the node has a <code>jsfid</code> attribute, and its value is
     * "ignore", child elements will be rendered as comment/verbatim
     * content.  The enclosing tag will not be rendered in the document,
     * only it's children. If the <code>jsfid</code> is "void",
     * the element will not be rendered but its children will
     * keep their original characteristics.</p>
     *
     * @param context commons chains
     * @return <code>true</code> if the command ends the chain
     * @exception Exception pushes an exception back to the invoking Command
     */
    public boolean execute(Context context) throws Exception {
        boolean isFinal = false;

        BuilderRuleContext builderRuleContext = (BuilderRuleContext) context;
        Node node = builderRuleContext.getNode();
        if (!node.isComment() && node.isStart() && node.getName() != null) {
            String jsfid = (String) node.getAttributes().get("jsfid");
            if (jsfid != null) {
               if (jsfid.equals("ignore")) {
                   builderRuleContext.setBuilder(BUILDERS[0]);
                   isFinal = true;
               } else if (jsfid.equals("void")) {
                   builderRuleContext.setBuilder(BUILDERS[1]);
                   isFinal = true;
               }
            }
        }

        return isFinal;
    }

}
